%%=========================================================================
% In this program, the map data of Univerisyt of Kansas in
% "KU_campus.geojson" to draw its 3D geoemtry. 
%https://overpass-turbo.eu/
%%=========================================================================
clear
clc
close all

% Default height of the bulding
z_space = 3;


% Only certain number of buildings in the file "KU_campus.geojson" is
% considered
% picked for drawing. 
Selected_building  = 1:40;
Excluded_Building  = [2 4 11 15 19  23   24  25  26  30  31 32 33 34 35  36];


% Used to store the bounday of the considered geometry
Coord_minx = 10^10;
Coord_miny = 10^10;
Coord_maxx = -10^10;
Coord_maxy = 0;


%% Extract coordinates of the buildings
fname = 'KU_campus.geojson';
val   = jsondecode(fileread(fname));
Temp  = val.features;
Bul_coords = cell(length(Selected_building)-length(Excluded_Building), 1);
count = 0;
for index  = 1:length(Selected_building)

    % only cosinder the building whose index is not in the set Excluded_Building
    B_id = Selected_building(index);   
    if( ismember(B_id,Excluded_Building) )
        continue;
    end

    % If the index-th objecti is not building, skip this it
    if(isfield(Temp(B_id).properties,'building')==0)
       continue; 
    end
    
    % building number +1
    count = count +1;

    % Number of larys for each building
    if(isfield(Temp(B_id).properties,'layer'))
        layer_char = Temp(B_id).properties.layer;
        layer = str2num(layer_char);
    elseif(isfield(Temp(B_id).properties,'building_levels'))
        layer_char = Temp(B_id).properties.building_levels;
        layer = str2num(layer_char);
    else
        % if the number of lays is not specified in the file
        % "KU_campus.geojson", we set its value as 3.
        layer = 3;
    end
    
    
    % x currently constains the longitude and altitude of the building. I need
    % to transform the  longitude and altitude into meters 
    x   = Temp(B_id).geometry.coordinates;  
    lon = x(:,:,1);    lat = x(:,:,2);
    Height = z_space*layer;  % Building height
    coordinates = [];        % Coordinates of points constructing this building
    for i = 1:length(lon)
       assert(lat(i)>=0 && lat(i)<=90);
       [x1,y1,z1]= lla2flat([lat(i), lon(i), 1000.0], [0.0, 0.0], 0.0, -100.0);  % transform the  longitude and altitude into meters
       z1 = Height;
       coordinates = [coordinates; x1,y1,z1];
    end
    
    
    %---Rotate the geometry to obtain a good coordinates and view-------------------
    % This is optional 
    theta= 0;
    coordinates(:,1) = coordinates(:,1)*cos(theta*pi/180) - coordinates(:,2)*sin(theta*pi/180);
    coordinates(:,2) = coordinates(:,1)*sin(theta*pi/180) + coordinates(:,2)*cos(theta*pi/180);
    
    coordinates(:,1) = -coordinates(:,1);
    %----------------------------------------------------------------------
    
    % Find the boundary of the overall geometry
    if(min(coordinates(:,1))<Coord_minx)
        Coord_minx = min(coordinates(:,1));
    end
    if(min(coordinates(:,2))<Coord_miny)
        Coord_miny = min(coordinates(:,2));
    end
    if(max(coordinates(:,1))>Coord_maxx)
        Coord_maxx = max(coordinates(:,1));
    end
    if(max(coordinates(:,2))>Coord_maxy)
        Coord_maxy = max(coordinates(:,2));
    end
    
    Bul_coords{count}=coordinates;
end


%% Use the extracted data to draw the 3D buildings
Resolution = 40;
Bul_coords_new = cell(length(Selected_building)-length(Excluded_Building), 1);
for bld_id = 1:size(Bul_coords,1)
    bld_id
    
    This_id      = cell2mat( Bul_coords(bld_id) );
    This_id(:,1) = This_id(:,1)-Coord_minx;
    This_id(:,2) = This_id(:,2)-Coord_miny;
    Bul_coords_new{bld_id} = This_id;
    
    % draw the walls specified by every two neighboring points so that we
    % obtain the outline of the building
    for i1 = 1:size(This_id,1)-1
        if( This_id(i1,1) == This_id(i1+1,1) )
           % vertical plane 
           X_set = ones(1,Resolution)*This_id(i1,1);
           Y_set = linspace(This_id(i1,2), This_id(i1+1,2),Resolution);
        elseif( This_id(i1,2) == This_id(i1+1,2) )
           % horizontal plane  
           X_set = linspace(This_id(i1,1), This_id(i1+1,1),Resolution); 
           Y_set = ones(1,Resolution)*This_id(i1,2);
        else
           X_set = linspace(This_id(i1,1), This_id(i1+1,1),Resolution);  
           k = (This_id(i1+1,2)-This_id(i1,2))/(This_id(i1+1,1)-This_id(i1,1));
           b = This_id(i1+1,2) - k*This_id(i1+1,1);
           Y_set = k*X_set+b;
        end

        z = [ones(size(X_set))*This_id(1,3); zeros(size(X_set))];    
        surf([X_set; X_set], [Y_set; Y_set], z)
        hold on
        %plot3(This_id(i1,1),This_id(i1,2),This_id(i1,3),'-ro')
        grid on
        
        axis equal
    end
end
xlabel('x')
ylabel('y')
save('data\Bul_coords_new','Bul_coords_new');
                            


%% Manully define the boundary points for the KU campus geometry 
% determine coordinates for the line connecting 3rd-4th points
k1 = (399.5011-414.0417)/(24.2772-0);
b1 = 340 - k1*60;  

% determine coordinates for the line connecting 4th-5th points
k2 = (448.452-499.47)/(35.111-52.44);
b2 = 390 - k2*((390-b1)/k1);

% determine coordinates for the line connecting 6th-7th points
k3 = (621.889-636.23)/(125.66-130.45);
b3 = 600 - k3*75;


x_boundary = [0  0   60   (390-b1)/k1  40         75   (700-b3)/k3   250   250  390  390 0];
y_boundary = [0  200 340  390          k2*40+b2   600  700           700   735  735  0   0];
plot(x_boundary,y_boundary,'-r');




%% Boundary of each subregion
% Line1_5   : [32 360]    [200 360]
% Line15_234: [200 0]     [200 700]
% Line2_3   : [200 300]   [380 300]
% Line3_4   : [200 520]   [380 520]
% plot([32 200],[360 360], '-r');
% hold on
% plot([200 200],[0 700], '-r');
% plot([200 380],[300 300], '-r');
% plot([200 380],[520 520], '-r');





% %% Plot the candidate BS locations in the geometry
% load data\BsCoord240.mat
% 
% plot3(BsCoord240(:,1),BsCoord240(:,2),BsCoord240(:,3),'bo')
% % for i=1:238
% %     plot3(BsCoord238(i,1),BsCoord238(i,2),BsCoord238(i,3),'bo')
% %     hold on
% % end
% 
% % load data\BsCoord184.mat
% % plot3(BsCoord184(:,1),BsCoord184(:,2),BsCoord184(:,3),'r*')
% 
% % for i=1:150
% %     plot3(BsCoord183(i,1),BsCoord183(i,2),BsCoord183(i,3),'r*')
% %     hold on
% % end
% 
% load data\BsCoord240.mat
% plot3(BsCoord240(:,1),BsCoord240(:,2),BsCoord240(:,3),'go')
% % for i=1:130
% %     plot3(BsCoord130(i,1),BsCoord130(i,2),BsCoord130(i,3),'g+')
% %     hold on
% % end

%% Grids : Partitation the geometry into many small grids
grid_center = [];
grid_centerWithRegionId = [];
UE_height    = 1.5; % Heigh of the UE
grid_len     = 5;   % side-length of each grid

for x = min(x_boundary):grid_len: (max(x_boundary)-grid_len)
    for y = min(y_boundary) :grid_len: (max(y_boundary)-grid_len)
        
        point = [ x+grid_len/2, y+grid_len/2, UE_height ];
        
        % Exclude the grid inside a building
        flag1 = 1;
        for bld_id = 1:size(Bul_coords,1)
            This_id = cell2mat( Bul_coords(bld_id) );
            This_id(:,1) = This_id(:,1)-Coord_minx;
            This_id(:,2) = This_id(:,2)-Coord_miny;
            idx = inpolygon(point(1),point(2),This_id(:,1),This_id(:,2)) ;
            if(idx==1)
               flag1 = 0;  % The gris is inside the builiding
               continue;
            end
        end
        
        %  Exclude the grid outside the considered geometry boundary
        flag2 = 2;
        idx = inpolygon(point(1),point(2), x_boundary, y_boundary) ;
        if(idx==0)
           flag2 = 0;  % The gris is outside the boundary
           continue;
        end
        
   
       % Plot all of the grids as the same kind
        if(flag1==1 && flag2==2)  
          %rectangle('Position',[point(1)-grid_len/2, point(2)-grid_len/2, grid_len,grid_len],'FaceColor','w')
          %hold on;
          grid_center = [grid_center; point];
          rectangle('Position',[point(1)-grid_len/2, point(2)-grid_len/2, grid_len,grid_len],'FaceColor','b'); 
        end
        
    end 
end


%% exclude some grids from the grid_center
% These grids are often at the corner of buildings and can be thought no UE in there
GridAtBuildingCorner = [153  217 340  341  396  455   580 610   611  655   682  1319   1735  1986 2059   2060   2061  2141  2142   2143  2144 2225    2476   2477  4375  5314   6672  6673  6674    6675];
ExcludedGridIs = [5104 GridAtBuildingCorner];
grid_center2 = [];
for i=1:size(grid_center,1)
    if(ismember(i,ExcludedGridIs))
        continue
    else
        grid_center2 = [grid_center2; grid_center(i,:)];
    end
end
grid_center = grid_center2;



%% Save the result
save('data\grid_center','grid_center')

